// ======================================================================
//
// ColorVector.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef COLOR_VECTOR_H
#define COLOR_VECTOR_H

// ======================================================================

class ColorVector
{
public:

	static const ColorVector fullyLit;
	static const ColorVector fullyDark;

private:

	real r, g, b, m;

	void computeMono(void);

public:

	ColorVector(void);
	explicit ColorVector(real newM);
	ColorVector(real newR, real newG, real newB);
	ColorVector(real newR, real newG, real newB, real newM);

	void makeZero(void);

	void setM(real newM);
	void setRGB(real newR, real newG, real newB);
	void setRGBM(real newM);
	void setRGBM(real newR, real newG, real newB);
	void setRGBM(real newR, real newG, real newB, real newM);

	real getR(void) const;
	real getG(void) const;
	real getB(void) const;
	real getM(void) const;

	void applyScale(real scale);
	void clampZeroToOne(void);

	ColorVector &operator +=(const ColorVector &v2);

	friend bool operator ==(const ColorVector &v1, const ColorVector &v2);
	friend bool operator !=(const ColorVector &v1, const ColorVector &v2);
};

// ======================================================================

inline void ColorVector::makeZero(void)
{
	r = CONST_REAL(0);
	g = CONST_REAL(0);
	b = CONST_REAL(0);
	m = CONST_REAL(0);
}

// ------------------------------------------------------------------------

inline void ColorVector::setM(real newM)
{
	m = newM;
}

// ------------------------------------------------------------------------

inline void ColorVector::setRGB(real newR, real newG, real newB)
{
	r = newR;
	g = newG;
	b = newB;
}
	
// ------------------------------------------------------------------------

inline void ColorVector::setRGBM(real newM)
{
	r = newM;
	g = newM;
	b = newM;
	m = newM;
}

// ------------------------------------------------------------------------

inline void ColorVector::setRGBM(real newR, real newG, real newB)
{
	r = newR;
	g = newG;
	b = newB;
	computeMono();
}

// ------------------------------------------------------------------------

inline void ColorVector::setRGBM(real newR, real newG, real newB, real newM)
{
	r = newR;
	g = newG;
	b = newB;
	m = newM;
}

// ------------------------------------------------------------------------

inline ColorVector::ColorVector(void)
{
	setRGBM(CONST_REAL(0));
}

// ------------------------------------------------------------------------

inline ColorVector::ColorVector(real newM)
{
	setRGBM(newM);
}

// ------------------------------------------------------------------------

inline ColorVector::ColorVector(real newR, real newG, real newB)
{
	setRGBM(newR, newG, newB);
}

// ------------------------------------------------------------------------

inline ColorVector::ColorVector(real newR, real newG, real newB, real newM)
{
	setRGBM(newR, newG, newB, newM);
}

// ------------------------------------------------------------------------

inline real ColorVector::getR(void) const
{
	return r;
}

// ------------------------------------------------------------------------

inline real ColorVector::getG(void) const
{
	return g;
}

// ------------------------------------------------------------------------

inline real ColorVector::getB(void) const
{
	return b;
}

// ------------------------------------------------------------------------

inline real ColorVector::getM(void) const
{
	return m;
}

// ------------------------------------------------------------------------

inline void ColorVector::clampZeroToOne(void)
{
	r = clamp(CONST_REAL(0), r, CONST_REAL(1));
	g = clamp(CONST_REAL(0), g, CONST_REAL(1));
	b = clamp(CONST_REAL(0), b, CONST_REAL(1));
	m = clamp(CONST_REAL(0), m, CONST_REAL(1));
}

// ------------------------------------------------------------------------

inline void ColorVector::applyScale(real scale)
{
	r *= scale;
	g *= scale;
	b *= scale;
	m *= scale;
}

// ------------------------------------------------------------------------

inline ColorVector &ColorVector::operator +=(const ColorVector &v2)
{
	r += v2.r;
	g += v2.g;
	b += v2.b;
	m += v2.m;
	return *this;
}

// ======================================================================

inline bool operator ==(const ColorVector &v1, const ColorVector &v2)
{
	return (v1.r == v2.r && v1.g == v2.g && v1.b == v2.b && v1.m == v2.m); //lint !e777 // testing float equality
}

// ------------------------------------------------------------------------

inline bool operator !=(const ColorVector &v1, const ColorVector &v2)
{
	return !(v1 == v2);
}

// ======================================================================

#endif
